@@ -10,7 +10,6 @@ extern crate pool;
 extern crate sha2;
 extern crate serde;
 extern crate serde_json;
-extern crate mio_uds;
 extern crate libc;
 extern crate time;
 extern crate poule;
@@ -25,3 +24,4 @@ pub mod channel;
 pub mod buffer;
 pub mod scm_socket;
 pub mod writer;
+pub mod ready;
